@@ -2,7 +2,7 @@
 //!
 //! This library provides an asynchronous, fully featured interface to the [Bluetooth Low Energy (BLE)](https://en.wikipedia.org/wiki/Bluetooth_Low_Energy)
 //! APIs of the [official Linux Bluetooth protocol stack (BlueZ)](http://www.bluez.org/).
-//! Both publishing local and consuming remote [GATT services](https://www.oreilly.com/library/view/getting-started-with/9781491900550/ch04.html) using *idiotmatic* Rust code is supported.
+//! Both publishing local and consuming remote [GATT services](https://www.oreilly.com/library/view/getting-started-with/9781491900550/ch04.html) using *idiomatic* Rust code is supported.
 //!
 //! This library depends on the [tokio] asynchronous runtime.
 //!
@@ -11,7 +11,7 @@
 //! * Bluetooth adapters
 //!     * enumeration
 //!     * configuration of power, discoverability, name, etc.
-//!     * hotplug support through change events stream
+//!     * hot-plug support through change events stream
 //! * Bluetooth devices
 //!     * discovery
 //!     * querying of address, name, class, signal strength (RSSI), etc.
@@ -367,9 +367,9 @@ pub use uuid::Uuid;
 /// Bluetooth error.
 #[derive(Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct Error {
-    /// Error kind
+    /// Error kind.
     pub kind: ErrorKind,
-    /// Message
+    /// Detailed error message provided by BlueZ.
     pub message: String,
 }
 
@@ -377,93 +377,91 @@ pub struct Error {
 #[derive(Clone, Debug, displaydoc::Display, Eq, PartialEq, Ord, PartialOrd, Hash, EnumString)]
 #[non_exhaustive]
 pub enum ErrorKind {
-    /// Bluetooth device already connected.
+    /// Bluetooth device already connected
     AlreadyConnected,
-    /// Bluetooth device already exists.
+    /// Bluetooth device already exists
     AlreadyExists,
-    /// Bluetooth authentication canceled.
+    /// Bluetooth authentication canceled
     AuthenticationCanceled,
-    /// Bluetooth authentication failed.
+    /// Bluetooth authentication failed
     AuthenticationFailed,
-    /// Bluetooth authentication rejected.
+    /// Bluetooth authentication rejected
     AuthenticationRejected,
-    /// Bluetooth authentication timeout.
+    /// Bluetooth authentication timeout
     AuthenticationTimeout,
-    /// Bluetooth connection attempt failed.
+    /// Bluetooth connection attempt failed
     ConnectionAttemptFailed,
-    /// Bluetooth device does not exist.
+    /// Bluetooth device does not exist
     DoesNotExist,
-    /// Bluetooth operation failed.
+    /// Bluetooth operation failed
     Failed,
-    /// Bluetooth operation in progress.
+    /// Bluetooth operation in progress
     InProgress,
-    /// Invalid arguments for Bluetooth operation.
+    /// Invalid arguments for Bluetooth operation
     InvalidArguments,
-    /// The data provided is of invalid length.
+    /// the data provided is of invalid length
     InvalidLength,
-    /// Bluetooth operation not available.
+    /// Bluetooth operation not available
     NotAvailable,
-    /// Bluetooth operation not authorized.
+    /// Bluetooth operation not authorized
     NotAuthorized,
-    /// Bluetooth device not ready.
+    /// Bluetooth device not ready
     NotReady,
-    /// Bluetooth operation not supported.
+    /// Bluetooth operation not supported
     NotSupported,
-    /// Bluetooth operation not permitted.
+    /// Bluetooth operation not permitted
     NotPermitted,
-    /// Invalid offset for Bluetooth GATT property.
+    /// invalid offset for Bluetooth GATT property
     InvalidOffset,
-    /// D-Bus error {0}.
-    #[strum(disabled)]
-    DBus(String),
-    /// Lost connection to D-Bus.
-    #[strum(disabled)]
-    DBusConnectionLost,
-    /// Invalid Bluetooth address: {0}.
+    /// invalid Bluetooth address: {0}
     #[strum(disabled)]
     InvalidAddress(String),
-    /// Invalid Bluetooth adapter name: {0}.
+    /// invalid Bluetooth adapter name: {0}
     #[strum(disabled)]
     InvalidName(String),
-    /// GATT services have not been resolved for that Bluetooth device.
+    /// GATT services have not been resolved for that Bluetooth device
     #[strum(disabled)]
     ServicesUnresolved,
-    /// Bluetooth application is not registered.
+    /// Bluetooth application is not registered
     #[strum(disabled)]
     NotRegistered,
-    /// The receiving Bluetooth device has stopped the notification session.
+    /// the receiving Bluetooth device has stopped the notification session
     #[strum(disabled)]
     NotificationSessionStopped,
-    /// The indication was not confirmed by the receiving device.
+    /// the indication was not confirmed by the receiving device
     #[strum(disabled)]
     IndicationUnconfirmed,
-    /// The target object was either not present or removed.
+    /// the target object was either not present or removed
     #[strum(disabled)]
     NotFound,
-    /// Internal error: {0}
+    /// internal error: {0}
     #[strum(disabled)]
     Internal(InternalErrorKind),
 }
 
 /// Internal Bluetooth error kind.
 ///
-/// This is most likely caused by incompatibilies between this library
+/// This is most likely caused by incompatibilities between this library
 /// and the version of the Bluetooth daemon.
 #[derive(Clone, Debug, displaydoc::Display, Eq, PartialEq, Ord, PartialOrd, Hash)]
 #[non_exhaustive]
 pub enum InternalErrorKind {
-    /// Invalid UUID: {0}.
+    /// invalid UUID: {0}
     InvalidUuid(String),
-    /// Invalid value.
+    /// invalid value
     InvalidValue,
-    /// Invalid modalias: {0}.
+    /// invalid modalias: {0}
     InvalidModalias(String),
-    /// Key {0} is missing.
+    /// key {0} is missing
     MissingKey(String),
-    /// Join error.
+    /// join error
     JoinError,
-    /// IO error {0:?}.
+    /// IO error {0:?}
     Io(std::io::ErrorKind),
+    /// D-Bus error {0}
+    DBus(String),
+    /// lost connection to D-Bus
+    DBusConnectionLost,
 }
 
 impl Error {
@@ -496,7 +494,7 @@ impl From<dbus::Error> for Error {
             .and_then(|s| ErrorKind::from_str(s).ok())
         {
             Some(kind) => kind,
-            _ => ErrorKind::DBus(err.name().unwrap_or_default().to_string()),
+            _ => ErrorKind::Internal(InternalErrorKind::DBus(err.name().unwrap_or_default().to_string())),
         };
         Self { kind, message: err.message().unwrap_or_default().to_string() }
     }
@@ -615,7 +613,7 @@ impl FromStr for Modalias {
     }
 }
 
-/// Gets all D-Bus objects from the bluez service.
+/// Gets all D-Bus objects from the BlueZ service.
 async fn all_dbus_objects(
     connection: &SyncConnection,
 ) -> Result<HashMap<Path<'static>, HashMap<String, PropMap>>> {
